	Program svp_standalone

!	This is a standalone program for the computation of SVP-radiative accelerations.
!	Description of the SVP method an download links can be found here: http://gradsvp.obspm.fr
!	This code has been written by G. Alecian (georges.alecian@obspm.fr).
!	Update: April 2020
!
! Details and discussion about the SVP method, and presentation of the physics used in 
! this program are in Alecian & LeBlanc, 2020 MNRAS (in preparation).

!	svp_standalone.f contains the main program and subroutines needed to 
!	compute SVP radiative accelerations in a standalone way and for some stars being
!	at a given stage of their evolution (at about the middle of the main-sequence), and
!	for which the corresponding model files are given in data_standalone/. sub-directory 
!	(see the readme.txt file).
!	This code uses also the routines that are in the files mod_donvar.f, mod_svp.f,
!	svp_codes.f. When the computation of radiative accelerations (using the SVP method) 
!	has to be implemented in an evolution code, the files mod_donvar.f, mod_svp.f,
!	svp_codes.f have to compiled and linked with the evolution code. The svp_standalone 
!	program shows how the svp subroutines have to called.

!	A previous version of the routines that are in the files mod_donvar.f, mod_svp.f,
!	svp_codes.f have been implemented in the following evolution codes in collaboration
!	with people named in parenthesis:
!	TGEC (S.Theado), CESAM2K (B. Pichon), CESTAM (M. Deal)

      USE MOD_SVP, Only: initialize_SVP, g_rad_SVP , grav, teff, mass,
     +                   id_ppxi, C_ref,isotops
      USE MOD_DONNEES, only: nomch => nom_chemin, ctes_94, nom_elem, 
     +                       nchim, nucleo, zi
      USE MOD_VARIABLES, ONLY : mtot => mstar , rtot => rstar

      Implicit None

***  variables that are read from .osc model files (output of CESTAM)

	integer  :: pzi,pnchim,lm
	parameter  (pzi=26,pnchim=20,lm=5000)

	real(KIND=KIND(0D0)) :: Mnucleo(pnchim),Mzi(pnchim),Mmtot,
     +	Mrtot,Mltot,Mteff,Mmass(lm),Mray(lm),Mt(lm),Mro(lm),
     +	Mkap(lm),Mgrad(lm),Mlum(lm),Mnel(lm),Mxchim(pnchim,lm),Mnp(lm)
	
	integer  :: itot,i,j,k, ios

	real(KIND=KIND(0D0)) :: ltot,ray,t,ro,kap,grad,lum,nel,
     + 		ychim(pnchim)
***

	character(1)   :: skip
	Character(4), Dimension(pnchim) :: nom_elem_bis

*======================================================
*** Input of subroutine initialize_SVP(...) in mod_svp.f
	character(64)           :: version_svp
*** Outputs of subroutine g_rad_SVP(...) in mod_svp.f
	Double Precision, Dimension(:),   Allocatable :: g_rad
	Double Precision, Dimension(:,:), Allocatable :: dg_rad
*====================================================== 
*** local variables
	integer         :: iunit,icouches,iso,nmod
	parameter(nmod=30)
	integer, dimension(nmod)   :: masses_list
	real     :: logT, logPop(0:pzi),temp(lm),Chi(pnchim,lm),khi(pnchim)
	real(KIND=KIND(0D0)) :: lolog,dpz
	integer	         :: massest,trouv
	character(64)        :: NFIa,NFIb,NFIm,version,teststring
      Character(80)        :: ligne
      Character(20),dimension(nmod) :: models_liste
*****************************************
2000	format(1x,1p8d10.3)
2001	format(1x,1p15d10.3)
2002	format(1x,i5,'	',f8.4,1p,60('	',d10.3))
2003	format(1x,'i	ychim	logT',30('	',a5,i3))
2004	format(1x,1p8d15.8)

	version='svp_standalone version 3.0, 24/04/2020, 12h21'
	print*, version

	NFIm='data_standalone/CESTAM_models/models_list.dat'
	open(unit=3,form='formatted',status='old',file=NFim)
	do i=1,nmod
		read(3,*,end=100) masses_list(i),models_liste(i)
		models_liste(i)=TRIM(ADJUSTL(models_liste(i)))
		print*,masses_list(i),'   ',models_liste(i)
	end do
100	close(3)
	
	print*, 'Choose a mass from the list (ex: 100=1Msol, 120=1.2Msol):'
	read(5,*) massest
	trouv=0
	do i=1,nmod
		if(massest.eq.masses_list(i)) then
			NFIb='data_standalone/CESTAM_models/'//TRIM(ADJUSTL(models_liste(i)))
			trouv=1
			exit
		endif
	end do
	if(trouv.eq.0) then
		print*,'Unidentified mass -> STOP'
		stop
	endif
	print*,'Chosen mass: ',massest,'	file: ',NFIb
	
	open(unit=1,form='formatted',status='old',file=NFib)
!**************************************** reads CESAM/CESTAM models (main-sequence star)
	call litcesam(
     &	nchim,nom_elem_bis,
     &	Mmtot,Mrtot,Mltot,Mteff,
     &	Mmass,Mray,Mt,Mro,Mkap,Mgrad,Mlum,Mnel,
     &	Mxchim,Mnp,itot)

! OUPUTS
! Elements for which SVP radiative acceleration have to be computed:
!   nchim: number of elements (2 isotopes of the same element being considered as 2 different elements)
!   nom_elem_bis: name of elements (ex: C12 for carbon 12, Al27 for aluminum 27)
! Stellar parameters:
!   Mmtot: mass of the star (g)
!   Mrtot: radius  of the star (cm)
!   Mltot: luminosity  of the star (erg)
!   Mteff: effective temperature
!   itot: total number of layers
! For each layer (arrays):
!   Mmass: mass above the layer (0 at the surface, 1 at the senter)
!   Mray: local radius
!   Mt:  temperature
!   Mro:  density
!   Mkap:  Rosseland opacity
!   Mgrad:  radiative temperature gradient
!   Mlum:  luminosity
!   Mnel: electron density  (number/cm3)
!   Mxchim: local abundances, mass fraction for each of the nchim elements
!   Mnp: proton density (number/cm3)
!****************************************

	print*,'Model read done !'

	OPEN (UNIT = 10,
     &        FILE = 'svp_standalone.job',
     &        STATUS = 'replace')

	write(10,*) version
	write(10,*) 
	write(10,*) 'Star mass (from model): ',Mmtot/1.9891d33,' Msol'
	write(10,*) 'Model file: ', NFIb

! output check
	OPEN (UNIT = 20,	
     &        FILE = 'ctrlg_rad.dat',
     &        STATUS = 'replace')

!	OPEN (UNIT = 30,
!     &        FILE = 'popul.dat',  ! ions population
!     &        STATUS = 'replace')

      Allocate( nom_elem(nchim) , nucleo(nchim) , zi(nchim) , Stat=ios )
      If ( ios /= 0 ) STOP " 1 "
	Allocate( g_rad(nchim+1) , dg_rad(nchim+1,nchim+1) , Stat=ios )
      If ( ios /= 0 ) STOP " 2 "

      nom_elem(1:nchim) = nom_elem_bis(1:nchim)

	write(10,*) 'Elements list:'
	write(10,91) nchim,(nom_elem(i),i=1,nchim)

	if(nchim.gt.pnchim) then
		print*, 'nchim>pnchim =>stop'
		stop
	end if

90	format(i3,14(1x,a4))
91	format(i3,14('	',a4))
92	format(i3,'	logt',60('	',i1,a6))

!**************************************** 
! INPUTS: nom_elem_bis, nchim

	call lisotop(nom_elem_bis,Mnucleo,Mzi,nchim)

! OUPUTS:
!   Mnucleo= atomic mass (12.0 for C12)
!   Mzi= atomic number (6 for C12, C13, C14)
!****************************************

	do i=1,nchim
		nucleo(i)=Mnucleo(i)
		zi(i)=Mzi(i)
	end do

	write(10,*) 'Atomic masses:'
	write(10,2001)(nucleo(i),i=1,nchim)

	write(10,*) 'Nucleus charges:'
	write(10,2001)(zi(i),i=1,nchim)

	mtot=Mmtot	! total mass of the star (g)
	rtot=Mrtot	! radius of the star(cm)
	ltot=Mltot	! luminosity of the star (Erg)
	teff=Mteff	! effective temperature (K)
	
	write(10,*) 
	write(10,*) 'mtot,rtot,ltot,teff:'
	write(10,2000) mtot,rtot,ltot,teff


      mtot = mtot * (1.98919000000001d0/1.9891d0) ! avoids a warning

	write(10,*) 

      CALL ctes_94 ()

	nomch='./'


!====================================================
! Initialization of the SVP calculation. This call 
! has to be done once, at the first step of diffusion calculation. This should  
! correspond to the arrival on the ZAMS.
! The only data used here are the total mass (mtot) of the star and the list of elements.
! They are used to find the right tables of SVP parameters, to load them
! for the needed elements,  and to interpolate them for mtot (this last feature is not used
! in this standalone program, since the masses of models are the same as those of 
! the provided SVP tables). All other data provided by the subroutine litcesam() called 
! above, can be updated at each time step of the evolution code, and be used for the 
! next call.

	version_svp='datai_SVP_v1'
      call initialize_SVP(version_svp)

!====================================================


! khi at surface
	write(10,'("List of khi, and",
     + " id_ppxi (=0 for element without SVP parameters): ")')
      print '(//)'
	print*,'Abundances: j,nom_elem,Mxchim,nucleo,ychim,C_ref'
	do j=1,nchim
		ychim(j)=Mxchim(j,1)/nucleo(j)
		if(id_ppxi(j).eq.1) then
			print'(i5,3x,a4,4es12.5)',j,ADJUSTL(nom_elem(j)),
     +		Mxchim(j,1),nucleo(j),ychim(j),C_ref(j)
		else
			print'(i5,3x,a4,3es12.5," NO SVP")',j,
     +		ADJUSTL(nom_elem(j)),Mxchim(j,1),nucleo(j),ychim(j)
		endif
	end do
	khi = 0.
	do j=1,nchim
		if(id_ppxi(j).eq.1) then
			do iso=1,nchim
				khi(j)=khi(j)+ychim(iso)/ychim(1)*isotops(j,iso)
			end do
			khi(j)=khi(j)/C_ref(j)
		else
			khi(j)=0.
		endif
		write(10,'(i2," khi_",a4," = ",es10.3,5x,i2)') j,
     +			ADJUSTL(nom_elem(j)),khi(j),id_ppxi(j)
	end do
	write(10,*)
	write(10,*) 'Accelerations, derivees et ychim:'

	write(10,92) nchim,
     + (id_ppxi(i),'*g'//ADJUSTL(nom_elem(i)),i=1,nchim),
     + (id_ppxi(i),'*d'//ADJUSTL(nom_elem(i)),i=1,nchim),
     + (id_ppxi(i),'*c'//ADJUSTL(nom_elem(i)),i=1,nchim)

! output de test
	write(20,93) ('lgr'//ADJUSTL(nom_elem(i)),
     +                'd'//ADJUSTL(nom_elem(i)),
     +                'chi'//ADJUSTL(nom_elem(i)),i=1,nchim)
93	format('logt',60('	',a7))

!	print*,'itot=',itot

! start loop layers. Here, there no time step. 

	do i=1,itot
		mass = Mmass(i)	! mass from radius ray to center (g)
		ray  = Mray(i)	! distance of the layer from the star center (cm)
		t    = Mt(i)	! temperature (K)
		ro   = Mro(i)	! mass density (g cm-3)
		kap  = Mkap(i)	! Rosseland opacity
		grad = Mgrad(i)	! temperature radiative gradient
		lum  = Mlum(i)	! star luminosity (erg)
		nel  = Mnel(i)	! electrons density in number
		temp(i)=log10(t)
		do j=1,nchim
			ychim(j) = Mxchim(j,i)/nucleo(j)
			Chi(j,i) = (ychim(j)/ychim(1))/C_ref(j)
		end do
! Defined in modules: nchim, nom_elem, nucleo, mtot, rtot
!====================================================

! computation of g_rad (after the 2nd call, g_rad_SVP computes g_rad)
		Call g_rad_SVP(ray,t,nel,grav,ychim,g_rad,dg_rad)

!====================================================

	  	 write(10,2002) i,temp(i),
     +		(g_rad(j),j=1,nchim),(dg_rad(j,j),j=1,nchim),
     +		(ychim(j),j=1,nchim)

	   	write(20,'(f8.4,1p,60("	",d12.5))') temp(i),
     +		(lolog(g_rad(j)),dg_rad(j,j),Chi(j,i),j=1,nchim)

	end do

! end loop layers

10	close(unit=1)

	print*,'number of layers: ',itot
	write(10,*)'number of layers: ',itot

      Deallocate( nom_elem )
      
      print '(//)'
      print *,'Accelerations are computed and are in the file ctrlg_rad.dat !'
      print '(//)'

	STOP " End standalone "

	end

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	function lolog(A)

	real(KIND=KIND(0D0))   :: lolog,A

	if (A >0.) then
		lolog = log10(A)
	else
		lolog = -30.
	end if

	return
	end
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine litcesam(
     &	nchim,nom_elem,
     &	mtot,rtot,ltot,teff,
     &	mass,ray,t,ro,kap,grad,lum,nel,
     &	xchim,np,itot)

! Reading of the model assigned to unit 1 (in main)

	implicit none

	integer pzi,pnchim,lglob,lm,lvar
	parameter (pzi=26,pnchim=20,lglob=50,lm=5000,lvar=100)

	integer	nchim,itot,j,k
! new .osc
	Integer, Parameter :: DP = KIND(1.d0)
	Integer :: i, nglob, nvar, nelem, niw, ios
	Real(DP), Dimension(lglob) :: glob
	Real(DP), Dimension(lvar,lm) :: var
	Real(DP), Dimension(lm) :: mue
	Real(DP)                :: somme,ceuler

	real*8	me,amu
	real*8	mtot,rtot,ltot,teff,
	1	mass(lm),ray(lm),t(lm),ro(lm),kap(lm),grad(lm),lum(lm),nel(lm),
	2	xchim(pnchim,lm),np(lm),xp(lm)
	
	character*1	skip
	character*4	nom_elem(pnchim)

!==============================================================================
	ceuler=2.718281828459045

	OPEN (UNIT = 10,
     &        FILE = 'lect_osc.job',
     &        STATUS = 'replace')


2000	format(1x,1p8d10.3)
2001	format(1x,1p15d10.3)
2002	format(1x,i4,1p,20('	',d11.4))
2004	format(1x,1p8d15.8)
	amu=1.6605402d-24	!masse atom. unite, Avogadro=1/amu
	me=9.1093897d-28	!masse electron


	read(1,'(a1)') skip
	read(1,'(a1)') skip
	read(1,'(a1)') skip
	read(1,'(a1)') skip
	read(1,90) nchim,(nom_elem(i),i=1,nchim)
	write(10,*) 'list considered elements:'
	write(10,91) nchim,(nom_elem(i),i=1,nchim)
*
	if(nchim.gt.pnchim) then
		print*, '(nchim.gt.pnchim) => STOP'
		stop
	end if

	Read(1,"(5I10)",IOStat=ios) itot, nglob, nvar, nelem, niw
	itot=itot-1 ! something works bad for the last layer. It is removed.
	print*,'itot, nglob, nvar, nelem, niw: ',itot, nglob, nvar, nelem, niw

!	glob: glogbal variables in mon_modele-ad.osc
!		glob(1)=mstar*msol
!		glob(2)=rtot*rsol
!		glob(3)=ltot*lsol
!		glob(4)=z0
!		glob(5)=x0
!		glob(6)=alpha
!		glob(7)=X dans ZC
!		glob(8)=Y dans ZC
!		glob(9)=d2p
!		glob(10)=d2ro
!		glob(11)=age
!		glob(12)=wrot vitesse de rotation globale
!		glob(13)=w_rot initial
!
!	var: variables per layer
!
!		var(1,i)=r*rsol
!	 	var(2,i)=log(m/mstar) -1.d38 au centre (en népérien?)
!		var(3,i)=t
!		var(4,i)=Ptot
!		var(5,i)=ro
!		var(6,i)=gradient reel d ln T / d ln P
!		var(7,i)=l
!		var(8,i)=kap
!		var(9,i)=energie thermo+gravifique
!		var(10,i)= Gamma1
!		var(11,i)= adiabatique gradient
!		var(12,i)=delta
!		var(13,i)=cp
!		var(14,i)=mu elec.
!		var(15,i)=vaissala, 0 at centre
!	 	var(16,i)=angular velocity, radian/sec
!	 	var(17,i)=d ln kappa / d ln T
!	 	var(18,i)=d ln kappa / d ln ro
!	 	var(19,i)=d epsilon(nuc) / d ln T
!	 	var(20,i)=d epsilon(nuc) / d ln ro
!		var(21,i)=Ptot / Pgas
!		var(22,i)= radiative gradient
!		...
!	  	var(nvar+j,i)=xchim(j), j=1,nchim 
! Ti47 is not titanium but all the remaining metals.
!
!---------------------------------------------------------------
!
	Read(1,"(5ES19.12)",IOStat=ios) (glob(i),i=1,nglob)
	
	mtot = glob(1)
	rtot = glob(2)
	ltot = glob(3)
	teff = (ltot/(4.*acos(-1.d0)*(rtot**2)*5.6692e-5))**(0.25)
	write(10,*) 'mtot,rtot,ltot,teff:'
	write(10,2000)mtot,rtot,ltot,teff
	
!	print*,'loop 1,itot=',itot
	Do i = 1 , itot
	   Read(1,"(5ES19.12)",IOStat=ios) (var(j,i),j=1,nvar+nchim)
	   If ( ios /= 0 ) STOP " Pb-09 "
	End Do
	print*

	write(10,'("i,mass,ray,t,ro,kap,grad,lum,nel")')
	Do i = 1 , itot   
	   ray(i)   = var(1,i)                !  cm
	   mass(i)  = 1. - ceuler**var(2,i)   ! (1-m/mtot) 
	   t(i)     = var(3,i)                !  K
	   ro(i)    = var(5,i)                !  g cm-3
	   lum(i)   = var(7,i)                !  erg
	   mue(i)   = var(14,i)               !  g mol-1  
	   nel(i)   = 6.0232e+23*ro(i)/mue(i) ! n_e = avogadro*dens / mu_e (cm-3)
	   grad(i)  = var(22,i)
	   kap(i)   = var(8,i)
	   write(10,2002)
     &	i,mass(i),ray(i),t(i),ro(i),kap(i),grad(i),lum(i),nel(i)
	End Do 
	
	write(10,'(//,"xchim:")')
	write(10,'(20("	",a4))')(nom_elem(i),i=1,nchim)
	Do i = 1 , itot
	   xchim(1:nchim,i) = var(nvar+1:nvar+nchim,i)  ! abundance: mass fraction of the element
	   somme = SUM(xchim(1:nchim,i))
	   If ( ABS( SUM(xchim(1:nchim,i)) - 1.0_DP ) >= 1.0e-13_DP ) 
     +	 Write(*,*) 'Sum xchim>1:',i,somme
	   write(10,2002) i, (xchim(j,i),j=1,nchim),somme
	   np(i)=xchim(1,i)*(ro(i)-nel(i)*me)/amu   ! a ce stade: np=nH
	End Do

	
	Do i = 1 , itot
		call  calcNP(t(i),nel(i),xp(i))
		np(i)=np(i)*xp(i)
	End Do


90	format(i3,14(1x,a4),/,14(1x,a4))
91	format(i3,14('	',a4),/,14('	',a4))


10	close(unit=1)
c	itot=i-1
	print*,'Layers number itot: ',itot
	close(unit=10)
	return

	end
!=======================================================================
	subroutine lisotop(nom_elem_bis,Mnucleo,Mzi,nchim)

! Reads isotopes list (isotopes.data) to connect with nom_elem

	Implicit None
	
	integer, parameter                      :: pzi=26,pnchim=20

	integer                                 :: nchim
	real(KIND=KIND(0D0)), dimension(pnchim) :: Mnucleo,Mzi
	Character(Len=4), dimension(pnchim)     :: nom_elem_bis

! Variables locales
	real(KIND=KIND(0D0)), dimension(200)  :: nucleoL,ziL
	Character(Len=4), dimension(200)      :: nom_elem_L
	
	integer                               :: i,is,j
	
	OPEN (UNIT = 2,
     +      FILE = 'data_standalone/isotopes.dat',
     +      STATUS = 'old')
	is = 0
	do i=1,200
		read(2,'(a4,d18.11,d5.0)',end=10) nom_elem_L(i),nucleoL(i),ziL(i)
		is=is+1
	end do
10	continue

	do j=1,nchim
		do i=1,is
			if(ADJUSTL(nom_elem_bis(j)).eq.nom_elem_L(i)) then
				Mnucleo(j)=nucleoL(i)
				Mzi(j)=ziL(i)
				cycle
			end if
		end do
	end do

	close(2)

	return
	end
!=======================================================================
	subroutine calcNP(temp,nel,xp)

! Computes H ionisation (taking density effect into acompte).
! From CESAM code,

	Implicit None
	Integer, Parameter :: DP = KIND(1.d0) , ndim = 10
!
	Real(DP), Parameter :: lambda_compton = 2426.310215e-13_DP    !  cm
	Real(DP), Parameter :: mec2 = 510998.9_DP                     !  eV
	Real(DP), Parameter :: k_eV = 8.617342e-05_DP                 ! n eV/K
!
!
	Real(DP), Parameter :: ioni_hydrogene = 13.595_DP 
	Real(DP), Parameter :: Rydberg        = 13.60569172_DP        ! eV
!
	Real(DP), Parameter :: Pi = 3.141592653589793238_DP
	Real(DP), Parameter :: avogadro = 6.022e+23_dp
!
	Real(DP) :: conversion, lambda_therm, expo, G = 1.0_DP , K    ! G ratio of statistical weights
!
! Corrections for EFF ionization : see Eggleton et al. A&A 23 (1973) 325
!
	Real(DP), Parameter :: z_barre_ini = 1.0_DP 
	Real(DP) :: z_barre, a0_cube, xhi0, delta_mu

! IN/OUT
	Real(DP) :: xp, temp, nel

	conversion = lambda_compton / SQRT( 2.0_DP * Pi * k_ev / mec2 )

	lambda_therm = conversion / SQRT( temp )
	K = lambda_therm**3 * nel / G
	expo = EXP( + ioni_hydrogene / ( k_ev * temp ) )
!
! estimation of initial correction to have z_barre value
!
	z_barre = z_barre_ini                  ! mean nucleus charge 
	a0_cube = (0.523e-8_DP/z_barre)**3     !  cm-3
	xhi0 = 13.5_DP * z_barre**2            !  eV
	delta_mu = a0_cube * nel * ( 1.0_DP + (20.0_DP*xhi0/k_ev) / temp )     ! correction EFF
	xp = 1.0_DP / ( 1.0_DP + K * expo * EXP( - delta_mu) )                 ! fraction of n(H)
!
! estimation of correction to have new z_barre value
!
	z_barre = 1.0_DP - xp
	a0_cube = (0.523e-8_DP/z_barre)**3     ! en cm-3
	xhi0 = 13.5_DP * z_barre**2            ! en eV
	delta_mu = a0_cube * nel * ( 1.0_DP + (20.0_DP*xhi0/k_ev) / temp )     ! correction EFF
	if(delta_mu.lt.300.) then   ! fraction n(H)
		xp = 1.0_DP / ( 1.0_DP + K * expo * EXP( - delta_mu) )  
!		print'(10es8.1)',K,expo, EXP( - delta_mu)
	else
		xp = 1.0_DP 
	endif
		               
	
!
! 
	return
	end
